# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .context_parameter import ContextParameter
from .for_loop_block import ForLoopBlock
import typing
from .workflow_definition_parameters_item import WorkflowDefinitionParametersItem
from .workflow_definition_blocks_item import WorkflowDefinitionBlocksItem
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class WorkflowDefinition(UniversalBaseModel):
    parameters: typing.List[WorkflowDefinitionParametersItem]
    blocks: typing.List[WorkflowDefinitionBlocksItem]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
