# This file was auto-generated by Fern from our API Definition.

import typing
from .non_empty_password_credential import NonEmptyPasswordCredential
from .non_empty_credit_card_credential import NonEmptyCreditCardCredential

CreateCredentialRequestCredential = typing.Union[NonEmptyPasswordCredential, NonEmptyCreditCardCredential]
